@@ -32,8 +32,6 @@
 #include <gnome.h>
 
 #include <regex.h>
-#include <gconf/gconf.h>
-#include <gconf/gconf-client.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
 #include <libgnomevfs/gnome-vfs-ops.h> 
 #include <bonobo-activation/bonobo-activation.h>
@@ -48,283 +46,9 @@
 #define ICON_THEME_EXECUTABLE_ICON "gnome-fs-executable"
 #define ICON_THEME_REGULAR_ICON    "gnome-fs-regular"
 
-/* START OF THE CDDB FUNCTIONS */
-
-static GtkCheckButtonClass *parent_class = NULL;
-
-struct _CDDBDisclosurePrivate {
-	GtkWidget *container;
-	char *shown;
-	char *hidden;
-	
-	guint32 expand_id;
-	GtkExpanderStyle style;
-
-	int expander_size;
-	int direction;
-};
-
-static void
-finalize (GObject *object)
-{
-	CDDBDisclosure *disclosure;
-
-	disclosure = CDDB_DISCLOSURE (object);
-	if (disclosure->priv == NULL) {
-		return;
-	}
-
-	g_free (disclosure->priv->hidden);
-	g_free (disclosure->priv->shown);
-
-	if (disclosure->priv->container != NULL) {
-		g_object_unref (G_OBJECT (disclosure->priv->container));
-	}
-	
-	g_free (disclosure->priv);
-	disclosure->priv = NULL;
-
-	G_OBJECT_CLASS (parent_class)->finalize (object);
-}
-
-static void
-get_x_y (CDDBDisclosure *disclosure,
-	 int *x,
-	 int *y,
-	 GtkStateType *state_type)
-{
-	GtkCheckButton *check_button;
-	int indicator_size, indicator_spacing;
-	int focus_width;
-	int focus_pad;
-	gboolean interior_focus;
-	GtkWidget *widget = GTK_WIDGET (disclosure);
-	GtkBin *bin = GTK_BIN (disclosure);
-	int width;
-	
-	if (GTK_WIDGET_VISIBLE (disclosure) &&
-	    GTK_WIDGET_MAPPED (disclosure)) {
-		check_button = GTK_CHECK_BUTTON (disclosure);
-		
-		gtk_widget_style_get (GTK_WIDGET (check_button),
-				      "indicator_size", &indicator_size,
-				      "indicator_spacing", &indicator_spacing,
-				      NULL);
-
-		gtk_widget_style_get (widget,
-				      "interior_focus", &interior_focus,
-				      "focus-line-width", &focus_width,
-				      "focus-padding", &focus_pad,
-				      NULL);
-		
-		*state_type = GTK_WIDGET_STATE (widget);
-		if ((*state_type != GTK_STATE_NORMAL) &&
-		    (*state_type != GTK_STATE_PRELIGHT)) {
-			*state_type = GTK_STATE_NORMAL;
-		}
-
-		if (bin->child) {
-			width = indicator_spacing * 3 + indicator_size ;
-		} else {
-			width = widget->allocation.width - 2 * GTK_CONTAINER (widget)->border_width;
-		}
-		
-		*x = widget->allocation.x + GTK_CONTAINER (widget)->border_width + (width) / 2;
-		*y = widget->allocation.y + widget->allocation.height / 2;
-
-		if (interior_focus == FALSE) {
-			*x += focus_width + focus_pad;
-		}
-
-		*state_type = GTK_WIDGET_STATE (widget) == GTK_STATE_ACTIVE ? GTK_STATE_NORMAL : GTK_WIDGET_STATE (widget);
-
-		if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL) {
-			*x = widget->allocation.x + widget->allocation.width - (*x - widget->allocation.x);
-		}
-	} else {
-		*x = 0;
-		*y = 0;
-		*state_type = GTK_STATE_NORMAL;
-	}
-}
-
-static gboolean
-expand_collapse_timeout (gpointer data)
-{
-	GtkWidget *widget = data;
-	CDDBDisclosure *disclosure = data;
-	GtkStateType state_type;
-	int x, y;
-	
-	gdk_window_invalidate_rect (widget->window, &widget->allocation, TRUE);
-	get_x_y (disclosure, &x, &y, &state_type);
-	
-	gtk_paint_expander (widget->style,
-			    widget->window,
-			    state_type,
-			    &widget->allocation,
-			    widget,
-			    "disclosure",
-			    x, y,
-			    disclosure->priv->style);
-
-	disclosure->priv->style += disclosure->priv->direction;
-	if ((int) disclosure->priv->style > (int) GTK_EXPANDER_EXPANDED) {
-		disclosure->priv->style = GTK_EXPANDER_EXPANDED;
-
-		if (disclosure->priv->container != NULL) {
-			gtk_widget_show (disclosure->priv->container);
-		}
-
-		g_object_set (G_OBJECT (disclosure),
-			      "label", disclosure->priv->hidden,
-			      NULL);
-			      
-		constraint_entry_changed_cb (NULL, NULL);	      
-			      
-		return FALSE;
-	} else if ((int) disclosure->priv->style < (int) GTK_EXPANDER_COLLAPSED) {
-		disclosure->priv->style = GTK_EXPANDER_COLLAPSED;
-
-		if (disclosure->priv->container != NULL) {
-			gtk_widget_hide (disclosure->priv->container);
-		}
-
-		g_object_set (G_OBJECT (disclosure),
-			      "label", disclosure->priv->shown,
-			      NULL);
-
-		constraint_entry_changed_cb (NULL, NULL);
-
-		return FALSE;
-	} else {
-		return TRUE;
-	}
-}
-
-static void
-do_animation (CDDBDisclosure *disclosure,
-	      gboolean opening)
-{
-	if (disclosure->priv->expand_id > 0) {
-		gtk_timeout_remove (disclosure->priv->expand_id);
-	}
-
-	disclosure->priv->direction = opening ? 1 : -1;
-	disclosure->priv->expand_id = g_timeout_add (50, expand_collapse_timeout, disclosure);
-}
-
-static void
-toggled (GtkToggleButton *tb)
-{
-	CDDBDisclosure *disclosure;
-
-	disclosure = CDDB_DISCLOSURE (tb);
-	do_animation (disclosure, gtk_toggle_button_get_active (tb));
-
-	if (disclosure->priv->container == NULL) {
-		return;
-	}
-}
-
-static void
-draw_indicator (GtkCheckButton *check,
-		GdkRectangle *area)
-{
-	GtkWidget *widget = GTK_WIDGET (check);
-	CDDBDisclosure *disclosure = CDDB_DISCLOSURE (check);
-	GtkStateType state_type;
-	int x, y;
-
-	get_x_y (disclosure, &x, &y, &state_type);
-	gtk_paint_expander (widget->style,
-			    widget->window,
-			    state_type,
-			    area,
-			    widget,
-			    "treeview",
-			    x, y,
-			    disclosure->priv->style);
-}
-
-static void
-class_init (CDDBDisclosureClass *klass)
-{
-	GObjectClass *object_class;
-	GtkWidgetClass *widget_class;
-	GtkCheckButtonClass *button_class;
-	GtkToggleButtonClass *toggle_class;
-	
-	object_class = G_OBJECT_CLASS (klass);
-	widget_class = GTK_WIDGET_CLASS (klass);
-	button_class = GTK_CHECK_BUTTON_CLASS (klass);
-	toggle_class = GTK_TOGGLE_BUTTON_CLASS (klass);
-	
-	toggle_class->toggled = toggled;
-	button_class->draw_indicator = draw_indicator;
-
-	object_class->finalize = finalize;
-
-	parent_class = g_type_class_peek_parent (klass);
-
-	gtk_widget_class_install_style_property (widget_class,
-						 g_param_spec_int ("expander_size",
-								   _("Expander Size"),
-								   _("Size of the expander arrow"),
-								   0, G_MAXINT,
-								   10, G_PARAM_READABLE));
-}
-
-static void
-init (CDDBDisclosure *disclosure)
-{
-	disclosure->priv = g_new0 (CDDBDisclosurePrivate, 1);
-	disclosure->priv->expander_size = 10;
-}
-
-GType
-cddb_disclosure_get_type (void)
-{
-	static GType type = 0;
-
-	if (type == 0) {
-		GTypeInfo info = {
-			sizeof (CDDBDisclosureClass),
-			NULL, NULL, (GClassInitFunc) class_init, NULL, NULL,
-			sizeof (CDDBDisclosure), 0, (GInstanceInitFunc) init
-		};
-
-		type = g_type_register_static (GTK_TYPE_CHECK_BUTTON, "CDDBDisclosure", &info, 0);
-	}
-
-	return type;
-}
-
-GtkWidget *
-cddb_disclosure_new (const char *shown,
-		     const char *hidden)
-{
-	CDDBDisclosure *disclosure;
-
-	disclosure = g_object_new (cddb_disclosure_get_type (), "label", shown, NULL);
-
-	disclosure->priv->shown = g_strdup (shown);
-	disclosure->priv->hidden = g_strdup (hidden);
-	return GTK_WIDGET (disclosure);
-}
-
-void
-cddb_disclosure_set_container (CDDBDisclosure *disclosure,
-			       GtkWidget *container)
-{
-	g_object_ref (G_OBJECT (container));
-	disclosure->priv->container = container;
-}
 
 /* START OF THE GCONF FUNCTIONS */
 
-static GConfClient *global_gconf_client = NULL;
-
 gboolean
 gsearchtool_gconf_handle_error (GError **error)
 {
@@ -382,15 +106,69 @@ gsearchtool_gconf_get_string (const gchar *key)
 	return result;
 }
 
+gboolean 
+gsearchtool_gconf_get_boolean (const gchar *key)
+{
+	gboolean result;
+	GConfClient *client;
+	GError *error = NULL;
+	
+	g_return_val_if_fail (key != NULL, FALSE);
+	
+	client = gsearchtool_gconf_client_get_global ();
+	g_return_val_if_fail (client != NULL, FALSE);
+	
+	result = gconf_client_get_bool (client, key, &error);
+	
+	if (gsearchtool_gconf_handle_error (&error)) {
+		result = FALSE;
+	}
+	
+	return result;
+}
+
+void
+gsearchtool_gconf_set_boolean (const gchar *key, const gboolean flag)
+{
+	GConfClient *client;
+	GError *error = NULL;
+	
+	g_return_if_fail (key != NULL);
+	
+	client = gsearchtool_gconf_client_get_global ();
+	g_return_if_fail (client != NULL);
+	
+	gconf_client_set_bool (client, key, flag, &error);
+	
+	gsearchtool_gconf_handle_error (&error);
+}
+
 /* START OF GENERIC GNOME-SEARCH-TOOL FUNCTIONS */
 
 gboolean 
 is_path_hidden (const gchar *path)
 {
-	gint results;
+	gint results = FALSE;
+	gchar *sub_str;
 	gchar *hidden_path_substr = g_strconcat (G_DIR_SEPARATOR_S, ".", NULL);
 	
-	results = (g_strstr_len (path, strlen (path), hidden_path_substr) != NULL);
+	sub_str = g_strstr_len (path, strlen (path), hidden_path_substr);
+
+	if (sub_str != NULL) {
+		gchar *gnome_desktop_str = g_strconcat (G_DIR_SEPARATOR_S, ".gnome-desktop", G_DIR_SEPARATOR_S, NULL);
+
+		/* exclude the .gnome-desktop folder */
+		if (strncmp (sub_str, gnome_desktop_str, strlen (gnome_desktop_str)) == 0) {
+			sub_str++;
+			results = (g_strstr_len (sub_str, strlen (sub_str), hidden_path_substr) != NULL);
+		}
+		else {
+			results = TRUE;
+		}
+		
+		g_free (gnome_desktop_str);
+	}
+
 	g_free (hidden_path_substr);
 	return results;
 }
@@ -401,6 +179,18 @@ is_path_in_home_folder (const gchar *path)
 	return (g_strstr_len (path, strlen (g_get_home_dir ()), g_get_home_dir ()) != NULL);
 }
 
+gboolean 
+is_path_in_mount_folder (const gchar *path)
+{
+	return (g_strstr_len (path, strlen ("/mnt/"), "/mnt/") != NULL);
+}
+
+gboolean 
+is_path_in_tmp_folder (const gchar *path)
+{
+	return (g_strstr_len (path, strlen ("/tmp/"), "/tmp/") != NULL);
+}
+
 gboolean
 file_extension_is (const char *filename, 
 		   const char *ext)
@@ -471,6 +261,27 @@ escape_single_quotes (const gchar *string)
 	return g_string_free (gs, FALSE);
 }
 
+gchar *
+remove_mnemonic_character (const gchar *string)
+{
+	GString *gs;
+	gboolean first_mnemonic = TRUE;
+	
+	if (string == NULL) {
+		return NULL;
+	}
+	
+	gs = g_string_new ("");
+	for(; *string; string++) {
+		if ((first_mnemonic) && (*string == '_')) {
+			first_mnemonic = FALSE;
+			continue;
+		}
+		g_string_append_c(gs, *string);
+	}
+	return g_string_free (gs, FALSE);
+}
+
 gint
 count_of_char_in_string (const gchar *string, 
 			 const gchar q)
@@ -482,16 +293,6 @@ count_of_char_in_string (const gchar *string,
 	return cnt;
 }
 
-gchar *
-get_date (const time_t file_time_raw)
-{
-	struct tm *file_time;
-	const char *format = _("%Y%m%d%H%M%S");
-
-	file_time = localtime (&file_time_raw);
-	return strdup_strftime (format, file_time);
-}
-
 gchar *
 get_readable_date (const time_t file_time_raw)
 {
@@ -515,8 +316,12 @@ get_readable_date (const time_t file_time_raw)
 	g_date_free (today);
 	g_date_free (file_date);
 			
-	/* the format varies depending on age of file. */
 	if (file_date_age == 0)	{
+	/* Translators:  Below are the strings displayed in the 'Date Modified'
+	   column of the list view.  The format of this string can vary depending 
+	   on age of a file.  Please modify the format of the timestamp to match
+	   your locale.  For example, to display 24 hour time replace the '%-I' 
+	   with '%-H' and remove the '%p'.  (See bugzilla report #120434.) */
 		format = g_strdup(_("today at %-I:%M %p"));
 	} else if (file_date_age == 1) {
 		format = g_strdup(_("yesterday at %-I:%M %p"));
@@ -639,7 +444,7 @@ get_file_type_with_mime_type (const gchar *filename,
 		return (gchar *)gnome_vfs_mime_get_description (GNOME_VFS_MIME_TYPE_UNKNOWN);
 	}
 
-	if (g_file_test (filename, G_FILE_TEST_IS_SYMLINK) == TRUE) {
+	if (g_file_test (filename, G_FILE_TEST_IS_SYMLINK)) {
 		if (g_ascii_strcasecmp (mimetype, GNOME_VFS_MIME_TYPE_UNKNOWN) == 0) {
 			GnomeVFSFileInfo *file_info;
 		
@@ -672,7 +477,7 @@ get_file_icon_with_mime_type (const gchar *filename,
 	if (filename == NULL || mimetype == NULL) {
 		icon_name = g_strdup (ICON_THEME_REGULAR_ICON);
 	}
-	else if ((g_file_test (filename, G_FILE_TEST_IS_EXECUTABLE) == TRUE) &&
+	else if ((g_file_test (filename, G_FILE_TEST_IS_EXECUTABLE)) &&
 	         (g_ascii_strcasecmp (mimetype, "application/x-executable-binary") == 0)) {
 		icon_name = g_strdup (ICON_THEME_EXECUTABLE_ICON);
 	}
@@ -683,6 +488,7 @@ get_file_icon_with_mime_type (const gchar *filename,
 
 	icon_path = gnome_icon_theme_lookup_icon (interface.icon_theme, icon_name, ICON_SIZE, 
 						  &icon_data, &base_size);
+	g_free (icon_name);
 	return icon_path;	
 }
 
@@ -762,7 +568,7 @@ open_file_with_application (const gchar *filename)
 		gnome_execute_async(NULL, argc, argv);	
 		gnome_vfs_mime_application_free(mimeApp);
 		g_free (command_line);	
-		g_free(argv);
+		g_strfreev (argv);
 		return TRUE;
 	}
 	return FALSE;
@@ -774,7 +580,7 @@ launch_file (const gchar *filename)
 	gchar *mime_type = gnome_vfs_get_mime_type (filename);	
 	gboolean result = FALSE;
 	
-	if ((g_file_test (filename, G_FILE_TEST_IS_EXECUTABLE) == TRUE) &&
+	if ((g_file_test (filename, G_FILE_TEST_IS_EXECUTABLE)) &&
 	    (g_ascii_strcasecmp (mime_type, BINARY_EXEC_MIME_TYPE) == 0)) { 
 		result = g_spawn_command_line_async (filename, NULL);
 	}
@@ -782,3 +588,72 @@ launch_file (const gchar *filename)
 	g_free (mime_type);
 	return result;
 }
+
+gchar *
+gsearchtool_unique_filename (const gchar *dir,
+                             const gchar *suffix)
+{
+	const gint num_of_words = 12;
+	gchar      *words[] = {
+		    "foo",
+		    "bar",
+		    "blah",
+	   	    "gegl",
+		    "frobate",
+		    "hadjaha",
+		    "greasy",
+		    "hammer",
+		    "eek",
+		    "larry",
+		    "curly",
+		    "moe",
+		    NULL};
+	gchar      *retval = NULL;
+	gboolean   exists = TRUE;
+
+	while (exists) {
+		gchar *filename;
+		gint   rnd;
+		gint   word;
+
+		rnd = rand ();
+		word = rand () % num_of_words;
+
+		filename = g_strdup_printf ("%s-%010x%s",
+					    words [word],
+					    (guint) rnd,
+					    suffix);
+
+		g_free (retval);
+		retval = g_strconcat (dir, "/", filename, NULL);
+		exists = g_file_test (retval, G_FILE_TEST_EXISTS);
+		g_free (filename);
+	}
+	return retval;
+}
+
+GtkWidget *
+gsearchtool_button_new_with_stock_icon (const gchar *label, 
+                                        const gchar *stock_id)
+{
+	GtkWidget *align;
+	GtkWidget *button;
+	GtkWidget *hbox;
+	GtkWidget *image;
+	GtkWidget *l;
+	
+	/* Copied from eel-gtk-extensions. */
+	button = gtk_button_new ();
+	l = gtk_label_new_with_mnemonic (label);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (l), GTK_WIDGET (button));
+	image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_BUTTON);
+	hbox = gtk_hbox_new (FALSE, 2);
+	align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+	gtk_box_pack_end (GTK_BOX (hbox), l, FALSE, FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (button), align);
+	gtk_container_add (GTK_CONTAINER (align), hbox);
+	gtk_widget_show_all (align);
+
+	return button;
+}
